package Contact::Model::tshirt_author;
use strict; use warnings;

use base 'Gantry::Utils::DBIxClass', 'Exporter';

use Contact::Model::GEN::tshirt_author;

our $TSHIRT_AUTHOR = 'Contact::Model::tshirt_author';

our @EXPORT_OK = ( '$TSHIRT_AUTHOR' );

1;

=head1 NAME

Contact::Model::tshirt_author - model for tshirt_author table (stub part)

=head1 DESCRIPTION

This model inherits from its generated helper, which inherits from
Gantry::Utils::DBIxClass.  It was generated by Bigtop, but is
NOT subject to regeneration.

=cut
