# NEVER EDIT this file.  It was generated and will be overwritten without
# notice upon regeneration of this application.  You have been warned.
package Apps::Checkbook::GEN::NoOp;

use strict;
use warnings;

use Apps::Checkbook qw{
    -PluginNamespace=Apps::Checkbook::NoOp
    PluginQ PluginB
};

our @ISA = qw( Apps::Checkbook );

use JSON;
use Gantry::Utils::TablePerms;


#-----------------------------------------------------------------
# $self->namespace() or Apps::Checkbook->namespace()
#-----------------------------------------------------------------
sub namespace {
    return 'Apps::Checkbook::NoOp';
}

1;

=head1 NAME

Apps::Checkbook::GEN::NoOp - generated support module for Apps::Checkbook::NoOp

=head1 SYNOPSIS

In Apps::Checkbook::NoOp:

    use base 'Apps::Checkbook::GEN::NoOp';

=head1 DESCRIPTION

This module was generated by bigtop and IS subject to regeneration.
Use it in Apps::Checkbook::NoOp to provide the methods below.
Feel free to override them.

=head1 METHODS

=over 4

=item namespace


=back

=head1 AUTHOR

Generated by bigtop and subject to regeneration.

=cut

