# NEVER EDIT this file.  It was generated and will be overwritten without
# notice upon regeneration of this application.  You have been warned.
package Contact::Model::bday;
use strict; use warnings;

Contact::Model::bday->table   ( 'bday'     );
Contact::Model::bday->columns ( Primary   => qw/
    id subid
/ );

Contact::Model::bday->columns ( All       => qw/
    id
    subid
    contact
    bday
/ );

Contact::Model::bday->columns ( Essential => qw/
    id
    subid
    contact
    bday
/ );

Contact::Model::bday->has_a( contact => 'Contact::Model::number' );

sub get_foreign_display_fields {
    return [ qw(  ) ];
}

sub get_foreign_tables {
    return qw(
        Contact::Model::number
    );
}

sub foreign_display {
    my $self = shift;

}

1;

=head1 NAME

Contact::Model::GEN::bday - model for bday table (generated part)

=head1 DESCRIPTION

This model mixes into Contact::Model::bday,
because Class::DBI bindings don't really allow a choice.
It was generated by Bigtop, and IS subject to regeneration.

=head1 METHODS

You may use all normal Class::DBI::Sweet methods and the ones listed here:

=over 4

=item get_foreign_display_fields

=item get_foreign_tables

=item foreign_display

=back

=cut
