use strict;

use Test::More tests => 6;

use_ok( 'Billing' );
use_ok( 'Billing::Status' );
use_ok( 'Billing::Company' );
use_ok( 'Billing::Customer' );
use_ok( 'Billing::LineItem' );
use_ok( 'Billing::Invoice' );
