package Apps::Checkbook::SOAP;

use strict;
use warnings;

use base 'Apps::Checkbook::GEN::SOAP';

#-----------------------------------------------------------------
# $self->namespace(  )
#-----------------------------------------------------------------
# This method inherited from Apps::Checkbook::GEN::SOAP

#-----------------------------------------------------------------
# $self->get_soap_ops(  )
#-----------------------------------------------------------------
# This method inherited from Apps::Checkbook::GEN::SOAP

#-----------------------------------------------------------------
# $self->greet(  )
#-----------------------------------------------------------------
sub greet {
    my ( $self, $input ) = @_;
} # END greet

#-----------------------------------------------------------------
# $self->cube_root(  )
#-----------------------------------------------------------------
sub cube_root {
    my ( $self, $input ) = @_;
} # END cube_root


1;

=head1 NAME

Apps::Checkbook::SOAP - A controller in the Apps::Checkbook application

=head1 SYNOPSIS

This package is meant to be used in a stand alone server/CGI script or the
Perl block of an httpd.conf file.

Stand Alone Server or CGI script:

    use Apps::Checkbook::SOAP;

    my $cgi = Gantry::Engine::CGI->new( {
        config => {
            #...
        },
        locations => {
            '/someurl' => 'Apps::Checkbook::SOAP',
            #...
        },
    } );

httpd.conf:

    <Perl>
        # ...
        use Apps::Checkbook::SOAP;
    </Perl>

    <Location /someurl>
        SetHandler  perl-script
        PerlHandler Apps::Checkbook::SOAP
    </Location>

If all went well, one of these was correctly written during app generation.

=head1 DESCRIPTION

This module was originally generated by Bigtop.  But feel free to edit it.
You might even want to describe the table this module controls here.

=head1 METHODS

=over 4

=item greet

=item cube_root


=back


=head1 METHODS INHERITED FROM Apps::Checkbook::GEN::SOAP

=over 4

=item namespace

=item get_soap_ops


=back


=head1 DEPENDENCIES

    Apps::Checkbook
    Apps::Checkbook::GEN::SOAP

=head1 AUTHOR

Phil Crow, E<lt>mail@example.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2007 Phil Crow

All rights reserved.

=cut
