# NEVER EDIT this file.  It was generated and will be overwritten without
# notice upon regeneration of this application.  You have been warned.
package Apps::Checkbook::GEN::Trans::Action;

use strict;
use warnings;

use Apps::Checkbook qw{
    -PluginNamespace=Apps::Checkbook::Trans::Action
    PluginQ PluginA
};

our @ISA = qw( Apps::Checkbook );

use JSON;
use Gantry::Utils::TablePerms;

use Apps::Checkbook::Model::trans qw(
    $TRANS
);

#-----------------------------------------------------------------
# $self->form( $row )
#-----------------------------------------------------------------
sub form {
    my ( $self, $row ) = @_;

    my $selections = $TRANS->get_form_selections();

    return {
        name       => 'trans',
        row        => $row,
        fields     => [
            {
                display_size => 2,
                name => 'status',
                label => 'Status2',
                type => 'text',
                is => 'int',
            },
        ],
    };
} # END form

#-----------------------------------------------------------------
# $self->controller_config(  )
#-----------------------------------------------------------------
sub controller_config {
    my ( $self ) = @_;

    return {
        permissions => {
            bits  => 'crudcr-d-r--',
            group => ''
        },
    };
} # END controller_config

#-----------------------------------------------------------------
# $self->namespace() or Apps::Checkbook->namespace()
#-----------------------------------------------------------------
sub namespace {
    return 'Apps::Checkbook::Trans::Action';
}

1;

=head1 NAME

Apps::Checkbook::GEN::Trans::Action - generated support module for Apps::Checkbook::Trans::Action

=head1 SYNOPSIS

In Apps::Checkbook::Trans::Action:

    use base 'Apps::Checkbook::GEN::Trans::Action';

=head1 DESCRIPTION

This module was generated by bigtop and IS subject to regeneration.
Use it in Apps::Checkbook::Trans::Action to provide the methods below.
Feel free to override them.

=head1 METHODS

=over 4

=item form

=item controller_config

=item namespace


=back

=head1 AUTHOR

Generated by bigtop and subject to regeneration.

=cut

