package Bigtop::Example::Billing::Model::customer;
use strict; use warnings;

use base 'Gantry::Utils::DBIxClass', 'Exporter';

use Bigtop::Example::Billing::Model::GEN::customer;

our $CUSTOMER = 'Bigtop::Example::Billing::Model::customer';

our @EXPORT_OK = ( '$CUSTOMER' );

1;

=head1 NAME

Bigtop::Example::Billing::Model::customer - model for customer table (stub part)

=head1 DESCRIPTION

This model inherits from its generated helper, which inherits from
Gantry::Utils::DBIxClass.  It was generated by Bigtop, but is
NOT subject to regeneration.

=head1 METHODS (mixed in from Bigtop::Example::Billing::Model::GEN::customer)

You may use all normal Gantry::Utils::DBIxClass methods and the
ones listed here:

=over 4

=item get_foreign_display_fields

=item get_foreign_tables

=item foreign_display

=item table_name

=back

=cut
