package Contact::Model::bday;
use strict; use warnings;

use base 'Gantry::Utils::CDBI', 'Exporter';

use Contact::Model::GEN::bday;

our $BDAY = 'Contact::Model::bday';

our @EXPORT_OK = ( '$BDAY' );

1;

=head1 NAME

Contact::Model::bday - model for bday table (stub part)

=head1 DESCRIPTION

This model inherits from Gantry::Utils::CDBI and uses its generated
helper Contact::Model::GEN::bday.

It was generated by Bigtop, but is NOT subject to regeneration.

=cut
