# NEVER EDIT this file.  It was generated and will be overwritten without
# notice upon regeneration of this application.  You have been warned.
package Contact::Model::number;
use strict; use warnings;

Contact::Model::number->table   ( 'number'     );
Contact::Model::number->sequence( 'number_seq' );

Contact::Model::number->columns ( All       => qw/
    id
    name
    number
/ );

Contact::Model::number->columns ( Essential => qw/
    id
    name
    number
/ );


sub get_foreign_display_fields {
    return [ qw( name ) ];
}

sub get_foreign_tables {
    return qw(
    );
}

sub foreign_display {
    my $self = shift;

    my $name = $self->name() || '';

    return "$name";
}

1;

=head1 NAME

Contact::Model::GEN::number - model for number table (generated part)

=head1 DESCRIPTION

This model mixes into Contact::Model::number,
because Class::DBI bindings don't really allow a choice.
It was generated by Bigtop, and IS subject to regeneration.

=head1 METHODS

You may use all normal Class::DBI::Sweet methods and the ones listed here:

=over 4

=item get_foreign_display_fields

=item get_foreign_tables

=item foreign_display

=back

=cut
