package Contact::Model::tshirt_color;
use strict; use warnings;

use base 'Gantry::Utils::DBIxClass', 'Exporter';

use Contact::Model::GEN::tshirt_color;

our $TSHIRT_COLOR = 'Contact::Model::tshirt_color';

our @EXPORT_OK = ( '$TSHIRT_COLOR' );

1;

=head1 NAME

Contact::Model::tshirt_color - model for tshirt_color table (stub part)

=head1 DESCRIPTION

This model inherits from its generated helper, which inherits from
Gantry::Utils::DBIxClass.  It was generated by Bigtop, but is
NOT subject to regeneration.

=cut
