package Apps::Checkbook::Model::sch_name;
use strict; use warnings;

use base 'Gantry::Utils::CDBI', 'Exporter';

use Apps::Checkbook::Model::GEN::sch_name;

our $SCH_NAME = 'Apps::Checkbook::Model::sch_name';

our @EXPORT_OK = ( '$SCH_NAME' );

1;

=head1 NAME

Apps::Checkbook::Model::sch_name - model for sch_name table (stub part)

=head1 DESCRIPTION

This model inherits from Gantry::Utils::CDBI and uses its generated
helper Apps::Checkbook::Model::GEN::sch_name.

It was generated by Bigtop, but is NOT subject to regeneration.

=cut
