# NEVER EDIT this file.  It was generated and will be overwritten without
# notice upon regeneration of this application.  You have been warned.
package Apps::Checkbook::Model::sch_name;
use strict; use warnings;

Apps::Checkbook::Model::sch_name->table   ( 'sch.name'     );
Apps::Checkbook::Model::sch_name->sequence( 'sch.name_seq' );
Apps::Checkbook::Model::sch_name->columns ( Primary   => qw/
    id
/ );

Apps::Checkbook::Model::sch_name->columns ( All       => qw/
    id
    name
    number
/ );

Apps::Checkbook::Model::sch_name->columns ( Essential => qw/
    id
    name
    number
/ );


sub get_foreign_display_fields {
    return [ qw(  ) ];
}

sub get_foreign_tables {
    return qw(
    );
}

sub foreign_display {
    my $self = shift;

}

1;

=head1 NAME

Apps::Checkbook::Model::GEN::sch_name - model for sch_name table (generated part)

=head1 DESCRIPTION

This model mixes into Apps::Checkbook::Model::sch_name,
because Class::DBI bindings don't really allow a choice.
It was generated by Bigtop, and IS subject to regeneration.

=head1 METHODS

You may use all normal Class::DBI::Sweet methods and the ones listed here:

=over 4

=item get_foreign_display_fields

=item get_foreign_tables

=item foreign_display

=back

=cut
